import { useEffect } from 'react';
import { useQuery, useMutation } from '@apollo/client';
import { GET_GROUPS, GET_TASKS, GET_STEP, GET_RECENT_TASKS, GET_FREQUENT_TASKS } from '../graphql/queries';
import { PRINT_TASK, PRINT_STEP, CREATE_NOTE, CREATE_GROUP, CREATE_TASK, CREATE_STEP } from '../graphql/mutations';
import { useTaskStore } from '../stores/taskStore';
import type { GroupWithTasks, TaskWithSteps, StepWithNotes } from '../types';
import { doesExist, isArray } from '../utils/typeGuards';

interface GraphQLStep {
  id: number;
  name: string;
  instructions: string;
  taskId: number;
  order: number;
  notes: GraphQLNote[];
  print_count: number;
  last_printed_at: string | null;
}

interface GraphQLNote {
  id: number;
  content: string;
  created_at: string;
  updated_at: string;
  user: {
    id: number;
    name: string;
  };
}

interface GraphQLTask {
  id: number;
  name: string;
  groupId: number;
  steps: GraphQLStep[];
  notes: GraphQLNote[];
  print_count: number;
  last_printed_at: string | null;
}

interface GraphQLGroup {
  id: number;
  name: string;
  tasks: GraphQLTask[];
  created_at: string;
  updated_at: string;
}

function toStepWithNotes(step: GraphQLStep | null | undefined): StepWithNotes | null {
  if (!doesExist(step)) return null;

  return {
    id: step.id,
    name: step.name,
    instructions: step.instructions,
    task_id: step.taskId,
    order: step.order,
    print_count: step.print_count,
    last_printed_at: step.last_printed_at ? new Date(step.last_printed_at) : undefined,
    notes: isArray(step.notes) ? step.notes.map(note => ({
      id: note.id,
      content: note.content,
      created_at: new Date(note.created_at),
      updated_at: new Date(note.updated_at),
      created_by: note.user.id,
      user: {
        id: note.user.id,
        name: note.user.name,
      },
    })) : [],
  };
}

function toTaskWithSteps(task: GraphQLTask | null | undefined): TaskWithSteps | null {
  if (!doesExist(task)) return null;

  return {
    id: task.id,
    name: task.name,
    group_id: task.groupId,
    print_count: task.print_count,
    last_printed_at: task.last_printed_at ? new Date(task.last_printed_at) : undefined,
    steps: isArray(task.steps) ? task.steps.map(step => toStepWithNotes(step)).filter(doesExist) : [],
    notes: isArray(task.notes) ? task.notes.map(note => ({
      id: note.id,
      content: note.content,
      created_at: new Date(note.created_at),
      updated_at: new Date(note.updated_at),
      created_by: note.user.id,
      user: {
        id: note.user.id,
        name: note.user.name,
      },
    })) : [],
  };
}

function toGroupWithTasks(group: GraphQLGroup | null | undefined): GroupWithTasks | null {
  if (!doesExist(group)) return null;

  return {
    id: group.id,
    name: group.name,
    created_at: new Date(group.created_at),
    updated_at: new Date(group.updated_at),
    tasks: isArray(group.tasks) ? group.tasks.map(task => toTaskWithSteps(task)).filter(doesExist) : [],
  };
}

export function useTaskDataOptimized() {
  const {
    groups,
    recentTasks,
    frequentTasks,
    currentStep,
    isLoadingGroups,
    isLoadingRecentTasks,
    isLoadingFrequentTasks,
    isLoadingStep,
    selectedGroup,
    selectedTask,
    selectedStep,
    setGroups,
    setRecentTasks,
    setFrequentTasks,
    setCurrentStep,
    setLoadingGroups,
    setLoadingRecentTasks,
    setLoadingFrequentTasks,
    setLoadingStep,
    setSelectedGroup,
    setSelectedTask,
    setSelectedStep,
    updateTaskPrintCount,
    updateStepPrintCount,
    addNoteToStep,
    addTaskToGroup,
    addStepToTask,
    getTaskById,
    getStepById,
  } = useTaskStore();

  // Only fetch groups if we don't have them already
  const { data: groupsData, loading: groupsLoading } = useQuery(GET_GROUPS, {
    skip: groups.length > 0,
    fetchPolicy: 'cache-first',
  });

  // Only fetch recent tasks if we don't have them already
  const { data: recentTasksData, loading: recentTasksLoading } = useQuery(GET_RECENT_TASKS, {
    skip: recentTasks.length > 0,
    fetchPolicy: 'cache-first',
  });

  // Only fetch frequent tasks if we don't have them already
  const { data: frequentTasksData, loading: frequentTasksLoading } = useQuery(GET_FREQUENT_TASKS, {
    skip: frequentTasks.length > 0,
    fetchPolicy: 'cache-first',
  });

  // Only fetch tasks for a group if we don't have them already
  const { data: tasksData, loading: tasksLoading } = useQuery(GET_TASKS, {
    variables: { groupId: selectedGroup?.id },
    skip: !selectedGroup?.id || (selectedGroup && selectedGroup.tasks.length > 0),
    fetchPolicy: 'cache-first',
  });

  // Only fetch step details if we don't have them already
  const { data: stepData, loading: stepLoading } = useQuery(GET_STEP, {
    variables: { id: selectedStep?.id },
    skip: !selectedStep?.id || (selectedStep && selectedStep.notes.length > 0),
    fetchPolicy: 'cache-first',
  });

  // Mutations
  const [printTask] = useMutation(PRINT_TASK);
  const [printStep] = useMutation(PRINT_STEP);
  const [createNote] = useMutation(CREATE_NOTE);
  const [createGroup] = useMutation(CREATE_GROUP);
  const [createTask] = useMutation(CREATE_TASK);
  const [createStep] = useMutation(CREATE_STEP);

  // Update store when data is fetched
  useEffect(() => {
    if (groupsData?.groups && groupsData.groups.length > 0) {
      const processedGroups = groupsData.groups
        .map((group: GraphQLGroup) => toGroupWithTasks(group))
        .filter(doesExist);
      setGroups(processedGroups);
    }
  }, [groupsData, setGroups]);

  useEffect(() => {
    if (recentTasksData?.recentTasks && recentTasksData.recentTasks.length > 0) {
      const processedTasks = recentTasksData.recentTasks
        .map((task: GraphQLTask) => toTaskWithSteps(task))
        .filter(doesExist);
      setRecentTasks(processedTasks);
    }
  }, [recentTasksData, setRecentTasks]);

  useEffect(() => {
    if (frequentTasksData?.frequentTasks && frequentTasksData.frequentTasks.length > 0) {
      const processedTasks = frequentTasksData.frequentTasks
        .map((task: GraphQLTask) => toTaskWithSteps(task))
        .filter(doesExist);
      setFrequentTasks(processedTasks);
    }
  }, [frequentTasksData, setFrequentTasks]);

  // Update tasks in the store when fetched, but don't update selectedGroup
  useEffect(() => {
    if (tasksData?.tasks && tasksData.tasks.length > 0 && selectedGroup) {
      const processedTasks = tasksData.tasks
        .map((task: GraphQLTask) => toTaskWithSteps(task))
        .filter(doesExist);

      // Update the groups array with the new tasks for this group
      const updatedGroups = groups.map(group =>
        group.id === selectedGroup.id
          ? { ...group, tasks: processedTasks }
          : group
      );
      setGroups(updatedGroups);
    }
  }, [tasksData, selectedGroup?.id, groups, setGroups]);

  // Update step details in the store when fetched, but don't update selectedStep
  useEffect(() => {
    if (stepData?.step && selectedStep) {
      const processedStep = toStepWithNotes(stepData.step);
      if (processedStep) {
        setCurrentStep(processedStep);
      }
    }
  }, [stepData, selectedStep?.id, setCurrentStep]);

  // Update loading states
  useEffect(() => {
    setLoadingGroups(groupsLoading);
  }, [groupsLoading, setLoadingGroups]);

  useEffect(() => {
    setLoadingRecentTasks(recentTasksLoading);
  }, [recentTasksLoading, setLoadingRecentTasks]);

  useEffect(() => {
    setLoadingFrequentTasks(frequentTasksLoading);
  }, [frequentTasksLoading, setLoadingFrequentTasks]);

  useEffect(() => {
    setLoadingStep(stepLoading);
  }, [stepLoading, setLoadingStep]);

  // Create virtual groups for recent and frequent tasks
  const virtualGroups: GroupWithTasks[] = [
    {
      id: -1,
      name: 'Recent Tasks',
      created_at: new Date(),
      updated_at: new Date(),
      tasks: recentTasks,
    },
    {
      id: -2,
      name: 'Frequent Tasks',
      created_at: new Date(),
      updated_at: new Date(),
      tasks: frequentTasks,
    },
  ];

  const allGroups = [...virtualGroups, ...groups];

  const handlePrintTask = async (taskId: string, userId: string) => {
    if (!taskId || !userId) {
      console.error('Cannot print task: missing taskId or userId');
      return;
    }

    try {
      const result = await printTask({ variables: { id: taskId, userId } });
      if (result.data?.printTask) {
        const task = result.data.printTask;
        updateTaskPrintCount(
          parseInt(taskId),
          task.print_count,
          task.last_printed_at
        );
      }
    } catch (error) {
      console.error('Error printing task:', error);
    }
  };

  const handlePrintStep = async (stepId: string, userId: string) => {
    if (!stepId || !userId) {
      console.error('Cannot print step: missing stepId or userId');
      return;
    }

    try {
      const result = await printStep({ variables: { id: stepId, userId } });
      if (result.data?.printStep) {
        const step = result.data.printStep;
        updateStepPrintCount(
          parseInt(stepId),
          step.print_count,
          step.last_printed_at
        );
      }
    } catch (error) {
      console.error('Error printing step:', error);
    }
  };

  const handleAddNote = async (content: string) => {
    if (!selectedStep?.id) {
      console.error('Cannot add note: missing stepId');
      return;
    }

    try {
      const result = await createNote({
        variables: {
          content,
          stepId: String(selectedStep.id),
        },
      });

      if (result.data?.createNote) {
        const note = result.data.createNote;
        addNoteToStep(selectedStep.id, {
          id: note.id,
          content: note.content,
          created_at: new Date(note.created_at),
          updated_at: new Date(note.updated_at),
          created_by: note.user.id,
          user: {
            id: note.user.id,
            name: note.user.name,
          },
        });
      }
    } catch (error) {
      console.error('Error adding note:', error);
    }
  };

  const handleCreateGroup = async (name: string) => {
    if (!name) {
      console.error('Cannot create group: missing name');
      return;
    }

    try {
      const result = await createGroup({ variables: { name } });
      if (result.data?.createGroup) {
        const group = result.data.createGroup;
        const newGroup: GroupWithTasks = {
          id: group.id,
          name: group.name,
          created_at: new Date(group.created_at),
          updated_at: new Date(group.updated_at),
          tasks: [],
        };
        setGroups([...groups, newGroup]);
      }
    } catch (error) {
      console.error('Error creating group:', error);
    }
  };

  const handleCreateTask = async (name: string, groupId: string) => {
    if (!name || !groupId) {
      console.error('Cannot create task: missing name or groupId');
      return;
    }

    try {
      const result = await createTask({ variables: { name, groupId } });
      if (result.data?.createTask) {
        const task = result.data.createTask;
        const newTask: TaskWithSteps = {
          id: task.id,
          name: task.name,
          group_id: task.group_id,
          print_count: 0,
          steps: [],
          notes: [],
        };
        addTaskToGroup(parseInt(groupId), newTask);
      }
    } catch (error) {
      console.error('Error creating task:', error);
    }
  };

  const handleCreateStep = async (name: string, instructions: string, taskId: string, order: number) => {
    if (!name || !instructions || !taskId || order < 1) {
      console.error('Cannot create step: missing required fields');
      return;
    }

    try {
      const result = await createStep({ variables: { name, instructions, taskId, order } });
      if (result.data?.createStep) {
        const step = result.data.createStep;
        const newStep: StepWithNotes = {
          id: step.id,
          name: step.name,
          instructions: step.instructions,
          task_id: step.task_id,
          order: step.order,
          print_count: 0,
          notes: [],
        };
        addStepToTask(parseInt(taskId), newStep);
      }
    } catch (error) {
      console.error('Error creating step:', error);
    }
  };

  const loading = isLoadingGroups || isLoadingRecentTasks || isLoadingFrequentTasks || isLoadingStep || tasksLoading;

  return {
    groups: allGroups,
    step: currentStep,
    loading,
    selectedGroup,
    selectedTask,
    selectedStep,
    setSelectedGroup,
    setSelectedTask,
    setSelectedStep,
    handlePrintTask,
    handlePrintStep,
    handleAddNote,
    handleCreateGroup,
    handleCreateTask,
    handleCreateStep,
    getTaskById,
    getStepById,
  };
}