import { Box, Paper, Typography, IconButton, TextField, Button } from '@mui/material';
import ArrowBackIcon from '@mui/icons-material/ArrowBack';
import PrintIcon from '@mui/icons-material/Print';
import type { GroupWithTasks, TaskWithSteps, StepWithNotes } from '../types';
import { CreateButtons } from '../components/CreateButtons';
import { ColorPickerButton } from '../components/ColorPickerButton';
import { useUserSelection } from '../hooks/useUserSelection';

function isGroupView(selectedGroup?: GroupWithTasks) {
  return !selectedGroup;
}

function isTaskView(selectedGroup?: GroupWithTasks, selectedTask?: TaskWithSteps) {
  return selectedGroup && !selectedTask;
}

function isStepView(selectedTask?: TaskWithSteps, selectedStep?: StepWithNotes) {
  return selectedTask && !selectedStep;
}

function isGroupWithTasks(group: any): group is GroupWithTasks {
  return group && typeof group.id === 'string' && Array.isArray(group.tasks);
}

export function MobileLayout({
  groups = [],
  selectedGroup,
  selectedTask,
  selectedStep,
  onGroupSelect,
  onTaskSelect,
  onStepSelect,
  onBack,
  onPrintTask,
  onPrintStep,
  onAddNote
}: {
  groups: GroupWithTasks[];
  selectedGroup?: GroupWithTasks;
  selectedTask?: TaskWithSteps;
  selectedStep?: StepWithNotes;
  onGroupSelect: (group: GroupWithTasks | undefined) => void;
  onTaskSelect: (task: TaskWithSteps | undefined) => void;
  onStepSelect: (step: StepWithNotes | undefined) => void;
  onBack: () => void;
  onPrintTask?: (taskId: string, userId: string) => void;
  onPrintStep?: (stepId: string, userId: string) => void;
  onAddNote?: (content: string) => void;
}) {
  const { selectedUser } = useUserSelection();
  const groupList: GroupWithTasks[] = groups;
  const taskList: TaskWithSteps[] = (selectedGroup?.tasks as TaskWithSteps[]) ?? [];
  const stepList: StepWithNotes[] = (selectedTask?.steps as StepWithNotes[]) ?? [];

  return (
    <Box sx={{ height: '100vh', p: 2 }}>
      {/* Header with back button */}
      {selectedGroup && !selectedTask && (
        <Box sx={{ display: 'flex', alignItems: 'center', mb: 2 }}>
          <IconButton onClick={onBack} sx={{ mr: 1 }}>
            <ArrowBackIcon />
          </IconButton>
          <Typography variant="h6">{selectedGroup.name}</Typography>
        </Box>
      )}

      {selectedTask && !selectedStep && (
        <Box sx={{ display: 'flex', alignItems: 'center', mb: 2 }}>
          <IconButton onClick={onBack} sx={{ mr: 1 }}>
            <ArrowBackIcon />
          </IconButton>
          <Typography variant="h6">{selectedTask.name}</Typography>
          {onPrintTask && selectedUser && (
            <IconButton
              onClick={() => onPrintTask(String(selectedTask.id), String(selectedUser.id))}
              sx={{ ml: 'auto' }}
            >
              <PrintIcon />
            </IconButton>
          )}
        </Box>
      )}

      {selectedStep && (
        <Box sx={{ display: 'flex', alignItems: 'center', mb: 2 }}>
          <IconButton onClick={onBack} sx={{ mr: 1 }}>
            <ArrowBackIcon />
          </IconButton>
          <Typography variant="h6">{selectedStep.name}</Typography>
          {onPrintStep && selectedUser && (
            <IconButton
              onClick={() => onPrintStep(String(selectedStep.id), String(selectedUser.id))}
              sx={{ ml: 'auto' }}
            >
              <PrintIcon />
            </IconButton>
          )}
        </Box>
      )}

      {/* Content */}
      <Paper sx={{ p: 2, overflow: 'auto', height: 'calc(100vh - 80px)' }}>
        {isGroupView(selectedGroup) && (
          <>
            <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', mb: 2 }}>
              <Typography variant="h6">Groups</Typography>
              <Box sx={{ display: 'flex', gap: 1 }}>
                <ColorPickerButton />
                <CreateButtons type="group" />
              </Box>
            </Box>

            {/* Groups List */}
            {groupList.map((group) => (
              <Box
                key={group.id}
                sx={{
                  p: 2,
                  cursor: 'pointer',
                  // no bg color
                  '&:hover': { bgcolor: 'action.hover' },
                }}
                onClick={() => onGroupSelect(group)}
              >
                <Typography>{group.name}</Typography>
              </Box>
            ))}
          </>
        )}

        {isTaskView(selectedGroup, selectedTask) && isGroupWithTasks(selectedGroup) && (
          <>
            <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', mb: 2 }}>
              <Typography variant="h6">Tasks</Typography>
              {selectedGroup.id > 0 && <CreateButtons type="task" parentId={String(selectedGroup.id)} />}
            </Box>
            <Box>
              {taskList.map((task) => (
                <Box
                  key={task.id}
                  sx={{
                    p: 2,
                    cursor: 'pointer',
                    // no bg color
                    '&:hover': { bgcolor: 'action.hover' },
                    display: 'flex',
                    justifyContent: 'space-between',
                    alignItems: 'center',
                  }}
                >
                  <Box onClick={() => onTaskSelect(task)} sx={{ flex: 1 }}>
                    <Typography>{task.name}</Typography>
                  </Box>
                  {onPrintTask && selectedUser && (
                    <IconButton
                      size="small"
                      onClick={(e) => {
                        e.stopPropagation();
                        onPrintTask(String(task.id), String(selectedUser.id));
                      }}
                    >
                      <PrintIcon />
                    </IconButton>
                  )}
                </Box>
              ))}
            </Box>
          </>
        )}

        {isStepView(selectedTask, selectedStep) && (
          <>
            <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', mb: 2 }}>
              <Typography variant="h6">Steps</Typography>
              <CreateButtons type="step" parentId={String(selectedTask?.id)} />
            </Box>
            <Box>
              {stepList.map((step) => (
                <Box
                  key={step.id}
                  sx={{
                    p: 2,
                    cursor: 'pointer',
                    bgcolor: selectedStep?.id === step.id ? 'action.selected' : 'transparent',
                    display: 'flex',
                    justifyContent: 'space-between',
                    alignItems: 'center',
                  }}
                >
                  <Box onClick={() => onStepSelect(step)} sx={{ flex: 1 }}>
                    <Typography>{step.order}: {step.name}</Typography>
                  </Box>
                  {onPrintStep && selectedUser && (
                    <IconButton
                      size="small"
                      onClick={(e) => {
                        e.stopPropagation();
                        onPrintStep(String(step.id), String(selectedUser.id));
                      }}
                    >
                      <PrintIcon />
                    </IconButton>
                  )}
                </Box>
              ))}
            </Box>
          </>
        )}

        {selectedStep && (
          <>
            <Typography sx={{ mb: 4 }}>{selectedStep.instructions}</Typography>

            <Box sx={{ mb: 4 }}>
              <Typography variant="h6" sx={{ mb: 2 }}>
                Notes ({selectedStep.notes.length})
              </Typography>
              {selectedStep.notes.map((note) => (
                <Box key={note.id} sx={{ mb: 2 }}>
                  <Typography variant="subtitle2">{note.user?.name}</Typography>
                  <Typography>{note.content}</Typography>
                </Box>
              ))}
            </Box>

            <Box sx={{ mb: 4 }}>
              <TextField
                fullWidth
                multiline
                rows={4}
                placeholder="Add a note..."
                sx={{ mb: 2 }}
              />
              {onAddNote && (
                <Button variant="contained" onClick={() => onAddNote('New note')}>
                  Add Note
                </Button>
              )}
            </Box>
          </>
        )}
      </Paper>
    </Box>
  );
}
